@@ -1,11 +1,14 @@
-VERSION = "0.2.0"
-
+VERSION = "0.3.0"
 import shutil
-import os
+
 
 # Make HubitModel available as from hubit import model
 from .model import HubitModel, _CACHE_DIR
 
 
 def clear_hubit_cache():
+    """
+    Clear the cache for all models. Will delete all serialized model cache
+    from the disk.
+    """
     shutil.rmtree(_CACHE_DIR, ignore_errors=True)
